// takes node and edge data and out node and edge svgs

var React = require('react')
var r = require('r-dom')
var Reflux = require('reflux')

var colaStore = require('stores/cola-store')
var Actions = require('actions')

var debug = require('debug')('holodex:graph-view:graph')
var Transform = require('d3-transform')
var reduce = require('lodash.reduce')
var router = require('router')
var Node = require('./node-ui')
var EdgeGroup = require('./edge-group')

var transform = Transform()
  .translate(function (d) { return d.slice(0, 2)})
  .scale(function (d) { return d[2]  })

module.exports = React.createClass({

  mixins: [
    Reflux.listenTo(colaStore, 'onColaUpdate')
  ],

  propTypes: {
    model: React.PropTypes.object,
    route: React.PropTypes.object,
    Icon: React.PropTypes.object
  },

  getInitialState: function (options) {
    return {
      nodes: colaStore.getNodes(),
      links: colaStore.getLinks(),
      queries: { active: [], inactive: [], queryMap: {} },
      width: colaStore.getWidth(),
      height: colaStore.getHeight(),
      vector: colaStore.getVector(),
      dragging: false,
      dragStart: false,
      client: { x: null, y: null },
      center: [colaStore.getWidth() / 2, colaStore.getHeight() / 2]
    }
  },

  onColaUpdate: function (payload) {
    debug('payload:', payload)
    this.setState(reduce(payload, function (memo, val, key) {
      memo[key] = val
      return memo
    }, {}))
  },

  render: function () {
    // debug('props', this.props, this.state)
    var nodes = this.state.nodes
    var edges = this.state.links
    var transformString = transform(this.state.vector)
    debug('transformString', transformString)
    debug('state', this.state)

    return (
      r.svg(
        {
          // draggable: true, //NOTE React does not support native drag events on svgs
          className: 'graph-svg',
          width: this.state.width,
          height: this.state.height,
          style: { cursor: this.state.dragging ? 'move' : 'auto' },
          onMouseDown: this.handleMouseDown,
          onMouseMove: this.handleMouseMove,
          onMouseUp: this.handleMouseUp,
          onWheel: this.handleWheel
          // onDragStart: this.handleDragStart,
        },
        [
          r.g(
            {
              id: 'transform-layer',
              transform: transformString
            },
            [
              r.g({className: 'halo-group'}, [
                r(EdgeGroup,
                    {
                      queries: this.state.queries,
                      nodes: nodes,
                      edges: edges,
                      model: this.props.model,
                      route: this.props.route
                    }
                  )
                ]
              ),
              r.g({className: 'node-group'}, [
                nodes.map(function (node) {
                  // debug('node', node)
                  return r(Node,
                    {
                      node: node,
                      key: node.id,
                      Icon: this.props.Icon,
                      handleClick: this.handleAgentClick(node.data)
                    }
                  )
                }.bind(this))
              ]
            )
          ]
        )]
      )
    )
  },

  componentWillReceiveProps: function (nextProps) {
    debug('componentWillReceiveProps', {nextProps: nextProps}, this.state)
  },

  getHref: function (model) {
    return router.format({ id: model.id })
  },

  navigate: router.navigate.bind(router),

  handleAgentClick: function (model) {
    var self = this
    return function () {
      // self.props.setColaTick(false)
      debug('handleClick', model)
      self.navigate(self.getHref(model))
    }
  },

  handleMouseDown: function (e) {
    debug('mousedown', e.target.className.baseVal)
    if (e.target.className.baseVal === 'graph-svg') {
      this.setState({ dragging: true })
      Actions.dragStart({ origin: [e.clientX, e.clientY] })
    }
  },

  handleMouseMove: function (e) {
    if (this.state.dragging) {
      Actions.drag({ point: [e.clientX, e.clientY] })
    }
  },

  handleMouseUp: function (e) {
    debug('mouseup', e)
    if (this.state.dragging) {
      Actions.dragEnd({ vector: this.state.vector })
      this.setState({ dragging: false })
    }
  },

  handleWheel: function (e) {
    var DOMNode = React.findDOMNode(this)
    var point = DOMNode.createSVGPoint()
    var layer = DOMNode.getElementById('transform-layer')
    point.x = e.clientX
    point.y = e.clientY

    //convert mouse coordinates to svg coordinates
    var svgGlobal = point.matrixTransform(layer.getCTM().inverse())
    Actions.wheel({ point: [svgGlobal.x, svgGlobal.y], delta: e.deltaY })

  }
})
