var Path = require('path')
var Url = require('url')
var pathMatch = require('path-match')()
var matchPath = require('path-to-regexp').compile
var qs = require('qs')
var debug = require('debug')('router')
var assign = require('lodash.assign')

function Router (config) {
  this.config = config

  if (process.browser) {
    var Location = require('observ-location')
    this.location = Location(config)
  }

  this.match = ':collection/:id?/:relation?'
  this.pathToRoute = pathMatch(this.match)
  this.routeToPath = matchPath(this.match)
}

Router.prototype.navigate = function (route) {
  var path
  if (typeof route === 'string') {
    path = route
  } else {
    path = this.format(route)
  }
  debug('path', path)
  this.location.set(path)
}

Router.prototype.route = function (path, callback) {
  debug('route', path, callback)

  if (typeof callback === 'undefined') {
    callback = path
    path = null
  }

  if (path) {
    return callback(this.parse(path))
  } else {
    return this.location(function (href) {
      var path = Url.parse(href).path
      return callback(this.parse(path))
    }.bind(this))
  }
}

Router.prototype.parse = function (path) {
  path = path || ''

  debug('parse(', path, ')')

  // split path into pathname and search
  var pathSplit = path.split('?')
  var pathname = pathSplit[0] || '/'
  var search = pathSplit[1] || ''

  // remove prefix from pathname
  pathname = pathname.replace(this.config.prefix || '/', '')

  // parse path into route
  var route = this.pathToRoute(pathname)

  // default / to to /groups/0
  if (pathname === '') {
    route = {
      collection: 'groups',
      id: 'enspiral-foundation'
    }
  }

  // parse search into query
  route.query = qs.parse(search)
  route.search = search

  debug('route() ->', route)
  return route
}

var strip = function (re) {
  return function (path) {
    return path.replace(re, '')
  }
}
var stripTrailingSlashes = strip(/\/$/g)
var stripSlashes = strip(/^\//g)

Router.prototype.format = function (route) {
  route = route || {}

  debug('format(', route, ')')

  // id may contain other parts
  if (route.id) {
    var idSplit = stripSlashes(route.id).split('/')
    debug('idSplit', idSplit, idSplit.length)
    if (idSplit.length > 1 && idSplit.length <= 3) {
      route = {
        collection: idSplit[0],
        id: idSplit[1],
        relation: idSplit[2]
      }
      debug('reformat', route)
    }
  }

  // format route into pathname
  var pathname = this.routeToPath(route)
  debug('pathname', pathname)

  // strip trailing slashes
  pathname = stripTrailingSlashes(pathname)
  debug('pathname', pathname)

  // add prefix to path
  pathname = Path.join(this.config.prefix || '/', pathname)
  debug('pathname', pathname)
  var path
  if (route.query) {
    // parse query into search
    var search = qs.stringify(route.query || {})

    // join pathname and search into path
    path = pathname + '?' + search
  } else {
    path = pathname
  }

  debug('format() ->', path)
  return path
}

var config = require('config')

module.exports = new Router(assign({
  prefix: config.ui.url.pathname
}, config.ui.router))
